@@ -31,13 +31,13 @@
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 ################################################################################
+
 """
-
-ident = '$Id: Types.py,v 1.17 2004/09/11 03:03:33 warnes Exp $'
-from version import __version__
-
 from __future__ import nested_scopes
 
+ident = '$Id: Types.py 1496 2010-03-04 23:46:17Z pooryorick $'
+from version import __version__
+
 import UserList
 import base64
 import cgi
@@ -53,6 +53,10 @@ from NS        import NS
 from Utilities import encodeHexString, cleanDate
 from Config    import Config
 
+NaN = float('NaN')
+PosInf = float('Inf')
+NegInf = -PosInf
+
 ###############################################################################
 # Utility functions
 ###############################################################################
@@ -199,6 +203,10 @@ class stringType(anyType):
 
         return data
 
+    def _marshalData(self):
+        return self._data
+
+
 class untypedType(stringType):
     def __init__(self, data = None, name = None, attrs = None):
         stringType.__init__(self, data, name, 0, attrs)
@@ -588,9 +596,10 @@ class timeType(anyType):
     def _marshalData(self):
         if self._cache == None:
             d = self._data
-            s = ''
-
-            s = time.strftime("%H:%M:%S", (2000, 1, 1) + d + (1, 1, -1))
+            #s = ''
+            #
+            #s = time.strftime("%H:%M:%S", (0, 0, 0) + d + (0, 0, -1))
+            s = "%02d:%02d:%02d" % d
             f = d[2] - int(d[2])
             if f != 0:
                 s += ("%g" % f)[1:]
@@ -1114,7 +1123,7 @@ class intType(anyType):
 
         if type(data) not in (IntType, LongType) or \
             data < -2147483648L or \
-            data >  2147483647:
+            data >  2147483647L:
             raise ValueError, "invalid %s value" % self._type
 
         return data
@@ -1253,13 +1262,13 @@ class compoundType(anyType):
             self.__dict__.update(data)
 
     def _aslist(self, item=None):
-        if item:
+        if item is not None:
             return self.__dict__[self._keyord[item]]
         else:
             return map( lambda x: self.__dict__[x], self._keyord)
 
     def _asdict(self, item=None, encoding=Config.dict_encoding):
-        if item:
+        if item is not None:
             if type(item) in (UnicodeType,StringType):
                 item = item.encode(encoding)
             return self.__dict__[item]
@@ -1308,8 +1317,14 @@ class compoundType(anyType):
         else:
             self.__dict__[name][subpos] = value
 
-        self._keyord[pos] = name
-
+        # only add to key order list if it does not already 
+        # exist in list
+        if not (name in self._keyord):
+            if pos < len(x):
+                self._keyord[pos] = name
+            else:
+                self._keyord.append(name)
+              
 
     def _getItemAsList(self, name, default = []):
         try:
@@ -1408,13 +1423,13 @@ class arrayType(UserList.UserList, compoundType):
 
 
     def _aslist(self, item=None):
-        if item:
+        if item is not None:
             return self.data[int(item)]
         else:
             return self.data
 
     def _asdict(self, item=None, encoding=Config.dict_encoding):
-        if item:
+        if item is not None:
             if type(item) in (UnicodeType,StringType):
                 item = item.encode(encoding)
             return self.data[int(item)]
@@ -1520,7 +1535,9 @@ class arrayType(UserList.UserList, compoundType):
                 self._poss[i + 1] += 1
 
             if self._dims[-1] and self._poss[-1] >= self._dims[-1]:
-                self._full = 1
+                #self._full = 1
+                #FIXME: why is this occuring?
+                pass
 
     def _placeItem(self, name, value, pos, subpos, attrs = None):
         curpos = [0] * len(self._dims)
@@ -1549,6 +1566,16 @@ class arrayType(UserList.UserList, compoundType):
 
         a[curpos[0]] = value
 
+class mapType(arrayType):
+    _validURIs = ('http://xml.apache.org/xml-soap',)
+
+    def __init__(self, data = None, name = None, attrs = None,
+        offset = 0, rank = None, asize = 0, elemsname = None):
+        
+        arrayType.__init__(self, data, name, attrs, offset, rank, asize,
+            elemsname)
+        self._keyord=['key','value']
+
 class typedArrayType(arrayType):
     def __init__(self, data = None, name = None, typed = None, attrs = None,
         offset = 0, rank = None, asize = 0, elemsname = None, complexType = 0):
@@ -1635,7 +1662,7 @@ class MethodFailed(Exception):
 
 def simplify(object, level=0):
     """
-    Convert the SOAPpy objects and thier contents to simple python types.
+    Convert the SOAPpy objects and their contents to simple python types.
 
     This function recursively converts the passed 'container' object,
     and all public subobjects. (Private subobjects have names that
